/**
 * CCGIEnv<br>
 * A class to simplify writing CGI programs. It parses input and puts the name-value
 * pairs into data structures. It formats the output messages to be sent out from the
 * CGI program.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 27th June, 1999
 */

#ifndef CGIENV_H
#define CGIENV_H

#include "DataPair.h"

class CCGIEnv
{
public:
	CDataPair   *pPostData;
	CDataPair   *pQueryData;
	CDataPair   *pEnvironment;
	CDataPair   *pOutputHeaders;

	int setHTTPResponseCode(int iCode, const char *sStr);
    int setContentLength(int iNewLength);
    int setContentType(const char *sType);
    int setResponseHeader(const char *sName, const char *sValue);

    int accumulateResponse(void *pData, int iLen);
    int accumulateResponse(const char *pData);
	int sendResponse(void);
    int sendHeaders(void);
	int restoreSavedState(const char *sFileName);
	int saveCurrentState(const char *sFileName);

	int readPostData(void);
	int readCGIEnv(void);
    int readQueryData(void);

	    CCGIEnv();
	    ~CCGIEnv();

private:
    char        *sResp;
    int         iRespLen;
};

#endif /* CGIENV_H */
