/**
 * CDataPair<br>
 * A class to simplify storage, manipulation and retrieval of name-value pairs.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 27th June, 1999
 */


#ifndef DATAPAIR_H
#define DATAPAIR_H

struct stDataPair
{
    char                *pName;
    char                *pValue;
    struct stDataPair   *pNext;
};

class CDataPair
{
public:
	char *  getAsString         (void);
	int     getNumberOfPairs    (void);
	int     escapeData          (void);
	int     setNameValuePair    (const char *sName, const char *sValue);
	int     getValueFor         (const char *sName, char **sValue, int *piLength);
	int     getValueFor         (const char *sName, char *sValue, int iLength);
    int     getNameValueAt      (char *sName, char *sValue, int iLength, int iIndex);
	int     getNameValueAt      (char *sName, char **sValue, int *piLength, int iIndex);
	int     isPresent           (const char *sName);
	int     getLengthFor        (const char *);
	char *  getValueFor         (const char *);
	int     unescapeData        (void);
	int     parseData           (const char *sData);
    int     removeAll           (void);
	        CDataPair           ();
	virtual ~CDataPair          ();

private:
    struct stDataPair   *pList;
    int                 iNumElements;
    struct stDataPair * findEntry       (const char *sName);
    struct stDataPair * findEntry       (int iIndex);
};

#endif /* DATAPAIR_H */
