/**
 * CCGIEnv<br>
 * A class to simplify writing CGI programs. It parses input and puts the name-value
 * pairs into data structures. It formats the output messages to be sent out from the
 * CGI program.
 * <br>
 * Make sure UNIX is defined while compiling on UNIX platform.<br>
 *
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 27th June, 1999
 */


#ifdef UNIX
#include <unistd.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "CGIEnv.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCGIEnv::CCGIEnv()
{
    pOutputHeaders = new CDataPair();
    pEnvironment = NULL;
    pQueryData = NULL;
    pPostData = NULL;
    sResp = NULL;
    iRespLen = 0;
    setHTTPResponseCode(200, "OK");
    setContentType("text/html");
}

CCGIEnv::~CCGIEnv()
{
    if (pPostData)      delete pPostData;
    if (pQueryData)     delete pQueryData;
    if (pEnvironment)   delete pEnvironment;
    if (pOutputHeaders) delete pOutputHeaders;
    if (sResp)          free(sResp);
}

int CCGIEnv::readCGIEnv(void)
{
    char *pThisEnv, **envptr, *pDupEnv, *pName, *pVal;

    if (pEnvironment) delete pEnvironment;
    pEnvironment = new CDataPair();

    envptr = environ;
    for(pThisEnv = *envptr; pThisEnv; pThisEnv = *(++envptr))
    {
        pDupEnv = strdup(pThisEnv);
        pName = strtok(pDupEnv, "=");
        pVal = strtok(NULL, "&");
        if(pName && pVal)
        {
            pEnvironment->setNameValuePair(pName, pVal);
        }
        free(pDupEnv);
    }
    return 0;
}

int CCGIEnv::readPostData(void)
{
    char *pContentLen, *pContent;
    int  iContentLen;

    if (pPostData) delete pPostData;
    pPostData = new CDataPair();
    if(pContentLen = getenv("REQUEST_METHOD"))
    {
        if (strcmp(pContentLen, "GET"))
        {
            if(pContentLen = getenv("CONTENT_LENGTH"))
            {
                if (0 < (iContentLen = atoi(pContentLen)))
                {
                    if (pContent = (char *)malloc(iContentLen+1))
                    {
                        int iBytesRead, iRead;

                        for(iBytesRead=0; iBytesRead<iContentLen; )
                        {
                            if (0 < (iRead = fread(pContent+iBytesRead, iContentLen-iBytesRead, 1, stdin)))
                            {
                                iBytesRead += iRead*(iContentLen-iBytesRead);
                            }
                            if (iRead < 0) break;
                        }
                        pContent[iBytesRead] = 0;
                        pPostData->parseData(pContent);
                        pPostData->unescapeData();
                        free(pContent);
                    }
                }
            }
        }
    }
    return 0;
}

int CCGIEnv::readQueryData(void)
{
    char *pQueryStr;

    if (pQueryData) delete pQueryData;
    pQueryData = new CDataPair();
    if(pQueryStr = getenv("QUERY_STRING"))
    {
        pQueryData->parseData(pQueryStr);
        pQueryData->unescapeData();
    }
    return 0;
}

int CCGIEnv::saveCurrentState(const char *sFileName)
{
    FILE    *fp;
    char    *pStr;

    if(NULL == (fp = fopen(sFileName, "w")))
    {
        return -1;
    }

    if(pEnvironment && (0 < pEnvironment->getNumberOfPairs()))
    {
        pEnvironment->escapeData();
        pStr = pEnvironment->getAsString();
        fprintf(fp, "%d\n%s\n", strlen(pStr), pStr);
        free(pStr);
    }
    else
    {
        fprintf(fp, "0\n\n");
    }

    if(pStr = getenv("QUERY_STRING"))
    {
        fprintf(fp, "%d\n%s\n", strlen(pStr), pStr);
    }
    else
    {
        fprintf(fp, "0\n\n");
    }

    if(pOutputHeaders && (0 < pOutputHeaders->getNumberOfPairs()))
    {
        pOutputHeaders->escapeData();
        pStr = pOutputHeaders->getAsString();
        fprintf(fp, "%d\n%s\n", strlen(pStr), pStr);
        free(pStr);
    }
    else
    {
        fprintf(fp, "0\n\n");
    }

    if(pPostData && (0 < pPostData->getNumberOfPairs()))
    {
        pPostData->escapeData();
        pStr = pPostData->getAsString();
        fprintf(fp, "%d\n%s\n", strlen(pStr), pStr);
        free(pStr);
    }
    else
    {
        fprintf(fp, "0\n\n");
    }
    fclose(fp);
    return 0;
}

int CCGIEnv::restoreSavedState(const char *sFileName)
{
    FILE    *fp;
    int     iSize, iCurrSize, iRead, iTotalRead, iStrLen;
    char    *pData, sLen[32];

    if(NULL == (fp = fopen(sFileName, "r")))
    {
        return -1;
    }

    if(pEnvironment) delete pEnvironment;
    if(pQueryData) delete pQueryData;
    if(pOutputHeaders) delete pOutputHeaders;
    if(pPostData) delete pPostData;

    pEnvironment = new CDataPair();
    pQueryData = new CDataPair();
    pOutputHeaders = new CDataPair();
    pPostData = new CDataPair();
    iCurrSize = 0;
    pData = NULL;

    // read environment
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    sLen[sizeof(sLen)-1] = 0;
    sLen[strlen(sLen)] = 0;
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\n' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\r' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    iSize = atoi(sLen);
    if (iSize > 0)
    {
        if (iCurrSize < (iSize+1))  pData = (char *)realloc(pData, iCurrSize=(iSize+1));
        if(!pData)  return -1;
        for(iTotalRead=0; iTotalRead < iSize; iTotalRead+=iRead)
        {
            iRead = fread(pData+iTotalRead, iSize-iTotalRead, 1, fp);
            if (iRead > 0)  iRead *= (iSize-iTotalRead);
            if (0 > iRead)  break;
        }
        pData[iSize] = 0;
        pEnvironment->parseData(pData);
        pEnvironment->unescapeData();
    }

    // read the new line
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    // read query string
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    sLen[sizeof(sLen)-1] = 0;
    sLen[strlen(sLen)] = 0;
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\n' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\r' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    iSize = atoi(sLen);
    if (iSize > 0)
    {
        if (iCurrSize < (iSize+1))  pData = (char *)realloc(pData, iCurrSize=(iSize+1));
        if(!pData)  return -1;
        for(iTotalRead=0; iTotalRead < iSize; iTotalRead+=iRead)
        {
            iRead = fread(pData+iTotalRead, iSize-iTotalRead, 1, fp);
            if (iRead > 0)  iRead *= (iSize-iTotalRead);
            if (0 > iRead)  break;
        }
        pData[iSize] = 0;
        pQueryData->parseData(pData);
        pQueryData->unescapeData();
    }

    // read the new line
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    // read output headers
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    sLen[sizeof(sLen)-1] = 0;
    sLen[strlen(sLen)] = 0;
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\n' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\r' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    iSize = atoi(sLen);
    if (iSize > 0)
    {
        if (iCurrSize < (iSize+1))  pData = (char *)realloc(pData, iCurrSize=(iSize+1));
        if(!pData)  return -1;
        for(iTotalRead=0; iTotalRead < iSize; iTotalRead+=iRead)
        {
            iRead = fread(pData+iTotalRead, iSize-iTotalRead, 1, fp);
            if (iRead > 0)  iRead *= (iSize-iTotalRead);
            if (0 > iRead)  break;
        }
        pData[iSize] = 0;
        pOutputHeaders->parseData(pData);
        pOutputHeaders->unescapeData();
    }

    // read the new line
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    // read post data
    if (!fgets(sLen, sizeof(sLen)-1, fp))   return -1;
    sLen[sizeof(sLen)-1] = 0;
    sLen[strlen(sLen)] = 0;
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\n' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    if ((iStrLen = strlen(sLen)) > 0)
    {
        if ('\r' == sLen[iStrLen-1]) sLen[iStrLen-1] = 0;
    }
    iSize = atoi(sLen);
    if (iSize > 0)
    {
        if (iCurrSize < (iSize+1))  pData = (char *)realloc(pData, iCurrSize=(iSize+1));
        if(!pData)  return -1;
        for(iTotalRead=0; iTotalRead < iSize; iTotalRead+=iRead)
        {
            iRead = fread(pData+iTotalRead, iSize-iTotalRead, 1, fp);
            if (iRead > 0)  iRead *= (iSize-iTotalRead);
            if (0 > iRead)  break;
        }
        pData[iSize] = 0;
        pPostData->parseData(pData);
        pPostData->unescapeData();
    }

    if (pData) free(pData);
    fclose(fp);
    return 0;
}

int CCGIEnv::accumulateResponse(void *pData, int iLen)
{
    sResp = (char *) realloc(sResp, iRespLen+iLen);
    if (!sResp) return -1;
    memcpy(sResp+iRespLen, pData, iLen);
    iRespLen+=iLen;
    return 0;
}

int CCGIEnv::accumulateResponse(const char *pData)
{
    return accumulateResponse((void *)pData, strlen(pData)+1);
}

int CCGIEnv::sendResponse(void)
{
    int iWrote, iRet;
    fflush(stdout);

    for (iWrote=0; iWrote<iRespLen; iWrote+=iRet)
    {
        iRet = fwrite(sResp, iRespLen-iWrote, 1, stdout);
        if (iRet < 0) return -1;
        iRet *= (iRespLen-iWrote);
    }
    return 0;
}

int CCGIEnv::sendHeaders(void)
{
    char    *pVal, sName[256];
    int     iMaxHdrs, iIndex, iValLen;

    // get status
    if(pVal = pOutputHeaders->getValueFor("Status"))
    {
        printf("%s: %s\n", "Status", pVal);
    }

    // get location
    if(pVal = pOutputHeaders->getValueFor("Location"))
    {
        printf("%s: %s\n", "Location", pVal);
    }

    // get everything else
    iMaxHdrs = pOutputHeaders->getNumberOfPairs();
    pVal = NULL;
    iValLen = 0;
    for (iIndex=0; iIndex < iMaxHdrs; iIndex++)
    {
        if (0 == pOutputHeaders->getNameValueAt(sName, &pVal, &iValLen, iIndex))
        {
            if (strcmp(sName, "Status") && strcmp(sName, "Location"))
            {
                printf("%s: %s\n", sName, pVal);
            }
        }
    }
    if(pVal) free(pVal);

    // print empty line
    printf("\n");

    fflush(stdout);
    return 0;
}

int CCGIEnv::setHTTPResponseCode(int iCode, const char *sStr)
{
    char sCode[1024];
    sprintf(sCode, "%d %s", iCode, sStr);
    return pOutputHeaders->setNameValuePair("Status", sCode);
}

int CCGIEnv::setContentLength(int iNewLength)
{
    char sLen[32];
    sprintf(sLen, "%d", iNewLength);
    return pOutputHeaders->setNameValuePair("Content-length", sLen);
}

int CCGIEnv::setContentType(const char *sType)
{
    return pOutputHeaders->setNameValuePair("Content-type", sType);
}

int CCGIEnv::setResponseHeader(const char *sName, const char *sValue)
{
    return pOutputHeaders->setNameValuePair(sName, sValue);
}
