/**
 * CGBHandler<br>
 * A class that contains routines to read and add to a data file containing the
 * guestbook entries.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 27th June, 1999
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "GBHandler.h"


CGBHandler::CGBHandler(const char *sFileNameIn)
{
    strcpy(sFileName, sFileNameIn);
    pTokenReplacer = NULL;
    pEntries = NULL;
    pDataParser = new CDataPair();
}


CGBHandler::~CGBHandler()
{
    if(pTokenReplacer)
    {
        delete pTokenReplacer;
    }
    freeEntries();
    if(pDataParser)
    {
        delete pDataParser;
    }
}

void CGBHandler::freeEntries(void)
{
    if(pEntries)
    {
        char **pCurrPtr;
        for(pCurrPtr=pEntries; pCurrPtr && *pCurrPtr; pCurrPtr++)
        {
            free(*pCurrPtr);
        }
        free(pEntries);
        pEntries = NULL;
    }
}

int CGBHandler::readEntries(int iNumber, int iLastIndex)
{
    FILE    *fp;
    int     iTotal, iBegin, iIndex;

    iTotal = countEntries();
    if ( (iLastIndex > iTotal) || (-1 == iLastIndex) )
    {
        iLastIndex = iTotal;
    }
    if (0 > (iBegin = iLastIndex-iNumber))
    {
        iBegin = 0;
    }

    if(NULL == (fp = fopen(sFileName, "r")))
    {
        return -1;
    }

    for(iIndex=0; iIndex < iBegin; iIndex++)
    {
        fgets(sOneEntry, sizeof(sOneEntry)-1, fp);
    }

    freeEntries();
    if (NULL == (pEntries = (char **)malloc((iLastIndex-iBegin+1)*sizeof(char *))))
    {
        fclose(fp);
        return -1;
    }

    for(; iIndex < iLastIndex; iIndex++)
    {
        fgets(sOneEntry, sizeof(sOneEntry)-1, fp);
        pEntries[iIndex-iBegin] = strdup(sOneEntry);
        pEntries[iIndex-iBegin+1] = NULL;
    }
    fclose(fp);
    return 0;
}

int CGBHandler::appendEntry(const char **ppFldValues)
{
    FILE        *fp;
    char        **ppThisTok, *sFileEntry, **ppFldNames;
    const char  **ppThisVal;

    ppFldNames = pTokenReplacer->getTokenList();

    pDataParser->removeAll();

    for(ppThisTok=ppFldNames, ppThisVal=ppFldValues; 
        ppThisTok && *ppThisTok && ppThisVal && *ppThisVal; 
        ppThisTok++, ppThisVal++)
    {
        if (!*ppThisVal || !strlen(*ppThisVal))
        {
            *ppThisVal = " ";
        }
        pDataParser->setNameValuePair(*ppThisTok, *ppThisVal);
    }
    pDataParser->escapeData();
    sFileEntry = pDataParser->getAsString();

    if(NULL == (fp = fopen(sFileName, "a")))
    {
        free(sFileEntry);
        return -1;
    }
    fwrite(sFileEntry, strlen(sFileEntry), 1, fp);
    fwrite("\n", 1, 1, fp);
    fclose(fp);
    free(sFileEntry);
    return 0;
}

int CGBHandler::prepareHTML(int iNumber, int iLastIndex, char **pBuffer, int *piLen)
{
    char    **pOneEntry;
    char    **ppValueList;
    int     iNumToks, iTokIndex;
    char    **ppTokList;

    if (0 != readEntries(iNumber, iLastIndex))
    {
        return -1;
    }

    ppTokList = pTokenReplacer->getTokenList();
    iNumToks  = pTokenReplacer->getNumTokens();

    for (pOneEntry=pEntries; pOneEntry && *pOneEntry; pOneEntry++);

    ppValueList = (char **)malloc((iNumToks+1)*sizeof(char *));
    for (pOneEntry--; (pOneEntry != (pEntries-1)) && pOneEntry && *pOneEntry; pOneEntry--)
    {
        pDataParser->removeAll();
        pDataParser->parseData(*pOneEntry);
        pDataParser->unescapeData();
        for(iTokIndex=0; iTokIndex < iNumToks; iTokIndex++)
        {
            ppValueList[iTokIndex] = pDataParser->getValueFor(ppTokList[iTokIndex]);
            if (!ppValueList[iTokIndex])
            {
                ppValueList[iTokIndex] = "";
            }
            ppValueList[iTokIndex+1] = 0;
        }
        pTokenReplacer->prepareHTML((const char **)ppValueList, pBuffer, piLen);
    }
    free(ppValueList);
    return 0;
}

int CGBHandler::setOutputTemplate(const char *sTemplateFile)
{
    if (pTokenReplacer) delete pTokenReplacer;
    pTokenReplacer = new CTokenReplacer(sTemplateFile);
    return 0;
}

int CGBHandler::countEntries(void)
{
    FILE    *fp;
    int     iCount;
    
    if(NULL == (fp = fopen(sFileName, "r")))
    {
        return -1;
    }

    for(iCount=0; !feof(fp); )
    {
        if (fgets(sOneEntry, sizeof(sOneEntry)-1, fp))
        {
            iCount++;
        }
    }
    fclose(fp);

    return iCount;
}

char ** CGBHandler::getTokenList(void)
{
    if (pTokenReplacer)
    {
        return pTokenReplacer->getTokenList();
    }
    return NULL;
}

int CGBHandler::getNumToks(void)
{
    if (pTokenReplacer)
    {
        return pTokenReplacer->getNumTokens();
    }
    return 0;
}

