/**
 * GuestBook<br>
 * A simple guest book for your home page.<br>
 * Implemented to demonstrate and test the capabilities of the CGI library 
 * CGI++. It requires two html template files - one partial html to format the 
 * appearance of each of the guest book entries as they will be displayed, and the 
 * other to display the guest book page as a whole. This application must be called
 * with the mode (GBMODE) as a query parameter. Valid values are &quot;add&quot; and 
 * &quot;show&quot;. The show mode displays the last 10 entries in the guest book and
 * the add mode adds a new entry to the guest book.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 27th June, 1999
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "CGIEnv.h"
#include "GBHandler.h"

#define GBMODENAME      "GBMODE"
#define GBVIEWNUMNAME   "GBVIEWNUM"
#define GBVIEWLASTNAME  "GBVIEWLAST"

#define GBFILENAME      "gbentries.log"
#define GBMSGTEMPLATE   "gbmessage.htt"
#define GBTEMPLATE      "gbtemplate.htt"
#define ERRTEMPLATE     "errtemplate.htt"
#define ADDMODE         "add"
#define SHOWMODE        "show"

#define MSGINVALIDMODE  "Invalid mode"
#define MSGERRPREPHTML  "Error preparing HTML"

CCGIEnv     *pCGIEnv;
CGBHandler  *pGBHandler;

static int displayMessages(void);
static int addEntry(void);
static int sendError(char *sErrStr);


int main(int argc, char **argv)
{
    char        *sMode;

    pCGIEnv = new CCGIEnv();

    pCGIEnv->readQueryData();
	pCGIEnv->readCGIEnv();
    pCGIEnv->readPostData();

    //uncomment this to save CGI state
    //pCGIEnv->saveCurrentState("cgicppsav.txt");
    
    //uncomment this to restore CGI state from file
    //pCGIEnv->restoreSavedState("cgicppsav.txt");
    
    sMode = pCGIEnv->pQueryData->getValueFor(GBMODENAME);

    if (!sMode)
    {
        sMode = "Empty";
        goto ERR_INVALID_MODE;
    }

    if (!strcmp(sMode, ADDMODE))
    {
        // add to guestbook and display updated messages
        pGBHandler = new CGBHandler(GBFILENAME);
        pGBHandler->setOutputTemplate(GBMSGTEMPLATE);
        addEntry();
        displayMessages();
        delete pGBHandler;
    }
    else if(!strcmp(sMode, SHOWMODE))
    {
        // display updated messages
        pGBHandler = new CGBHandler(GBFILENAME);
        pGBHandler->setOutputTemplate(GBMSGTEMPLATE);
        displayMessages();
        delete pGBHandler;
    }
    else
    {
        goto ERR_INVALID_MODE;
    }

    delete pCGIEnv;
    return 0;

ERR_INVALID_MODE:
    {
        char sMsg[256];
        sprintf(sMsg, "%s (%s)", MSGINVALIDMODE, sMode);
        sendError(sMsg);
    }
    delete pCGIEnv;
    return -1;
}


int displayMessages(void)
{
    char            *pMsgStr, *pRetHtml, *sNumber;
    int             iLen, iNumber, iLast;
    CTokenReplacer  *pTokenRepl;
    char            **pValList;

    // get the html snippet
    if (!(sNumber = pCGIEnv->pPostData->getValueFor(GBVIEWNUMNAME)))
    {
        if (!(sNumber = pCGIEnv->pQueryData->getValueFor(GBVIEWNUMNAME)))
        {
            sNumber = "10";
        }
    }
    iNumber = atoi(sNumber);

    if (!(sNumber = pCGIEnv->pPostData->getValueFor(GBVIEWLASTNAME)))
    {
        if (!(sNumber = pCGIEnv->pQueryData->getValueFor(GBVIEWLASTNAME)))
        {
            sNumber = "-1";
        }
    }
    iLast = atoi(sNumber);

    pMsgStr = NULL;
    iLen    = 0;
    if (0 != pGBHandler->prepareHTML(iNumber, iLast, &pMsgStr, &iLen))
    {
        pMsgStr = MSGERRPREPHTML;
    }

    // make the complete html
    pTokenRepl = new CTokenReplacer(GBTEMPLATE);
    pRetHtml = NULL;
    iLen = 0;
    if (!(pValList = (char **)malloc(2*sizeof(char *))))
    {
        delete pTokenRepl;
        return -1;
    }
    pValList[0] = pMsgStr;
    pValList[1] = 0;
    pTokenRepl->prepareHTML((const char **)pValList, &pRetHtml, &iLen);
    if (0 != pCGIEnv->accumulateResponse(pRetHtml))
    {
        delete pTokenRepl;
        free(pRetHtml);
        free(pMsgStr);
        free(pValList);
        return -1;
    }
    pCGIEnv->sendHeaders();
    pCGIEnv->sendResponse();
    delete pTokenRepl;
    free(pRetHtml);
    free(pMsgStr);
    free(pValList);
    return 0;
}


int addEntry(void)
{
    char    **ppTokList, **ppValList;
    int     iNumToks, iIndex;

    ppTokList = pGBHandler->getTokenList();
    iNumToks  = pGBHandler->getNumToks();
    if(!iNumToks || !ppTokList)
    {
        goto ERROR_NOFREE;
    }
    if (!(ppValList = (char **)malloc((iNumToks+1)*sizeof(char *))))
    {
        goto ERROR_NOFREE;
    }
    for(iIndex=0; iIndex<iNumToks; iIndex++)
    {
        ppValList[iIndex] = pCGIEnv->pQueryData->getValueFor(ppTokList[iIndex]);
        if (!ppValList[iIndex])
        {
            ppValList[iIndex] = pCGIEnv->pPostData->getValueFor(ppTokList[iIndex]);
        }
        if (!ppValList[iIndex])
        {
            ppValList[iIndex] = pCGIEnv->pEnvironment->getValueFor(ppTokList[iIndex]);
        }
        if (!ppValList[iIndex])
        {
            ppValList[iIndex] = " ";
        }
        ppValList[iIndex+1] = NULL;
    }

    if (0 != pGBHandler->appendEntry((const char **)ppValList))
    {
        goto ERROR;
    }

    free(ppValList);
    return 0;

ERROR:
    free(ppValList);
ERROR_NOFREE:
    return -1;
}


int sendError(char *sErrStr)
{
    char            *pRetHtml;
    int             iLen;
    CTokenReplacer  *pTokenRepl;
    char            **pValList;

    pTokenRepl = new CTokenReplacer(ERRTEMPLATE);
    pRetHtml = NULL;
    iLen = 0;
    if (!(pValList = (char **)malloc(2*sizeof(char *))))
    {
        delete pTokenRepl;
        return -1;
    }
    pValList[0] = sErrStr;
    pValList[1] = 0;
    pTokenRepl->prepareHTML((const char **)pValList, &pRetHtml, &iLen);
    if (0 != pCGIEnv->accumulateResponse(pRetHtml))
    {
        delete pTokenRepl;
        free(pRetHtml);
        free(pValList);
        return -1;
    }
    pCGIEnv->sendHeaders();
    pCGIEnv->sendResponse();
    delete pTokenRepl;
    free(pRetHtml);
    free(pValList);
    return 0;
}
