/**
 * CTokenReplacer<br>
 * A class to prepare complete HTML (or any other format) output from a template 
 * containing tokens to be replaced and the values that are to be replaced.
 * <br>
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 27th June, 1999
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "TokenReplacer.h"

char * CTokenReplacer::TOKEN_END = "]]]";
char * CTokenReplacer::TOKEN_START = "[[[";

CTokenReplacer::CTokenReplacer(const char *sFileName)
{
    sFileContent = NULL;
    slurpFile(sFileName, &sFileContent);
    parseTokens();
}

CTokenReplacer::~CTokenReplacer()
{
    if (sFileContent)   free(sFileContent);
}

char ** CTokenReplacer::getTokenList()
{
    return asTokenList;
}

void CTokenReplacer::prepareHTML(const char **sValueList, char **sReturn, int *piLen)
{
    int iValueIndex, iTextIndex;

    iValueIndex=0;

    for (iTextIndex=0; asTextList[iTextIndex]; iTextIndex++)
    {
        appendAndExpand(sReturn, asTextList[iTextIndex], piLen);
        if(sValueList[iValueIndex])
        {
            appendAndExpand(sReturn, sValueList[iValueIndex++], piLen);
        }
    }
    while(sValueList[iValueIndex])
    {
        appendAndExpand(sReturn, sValueList[iValueIndex++], piLen);
    }
}

void CTokenReplacer::appendAndExpand(char **ppTarget, const char *pAppend, int *piLen)
{
    int iExtraLen = strlen(pAppend);
    int iCurrLen = (*ppTarget) ? strlen(*ppTarget) : 0;

    if ((iCurrLen+iExtraLen) >= *piLen)
    {
        *piLen = iCurrLen+iExtraLen+256;
        *ppTarget = (char *)realloc(*ppTarget, *piLen);
        (*ppTarget)[iCurrLen] = 0;
    }
    strcat(*ppTarget, pAppend);
}

void CTokenReplacer::parseTokens(void)
{
    int iTextIndex=0, iTokenIndex=0;
    char *pStart, *pEnd, *pCurrPos;

    if (!sFileContent)
    {
        asTextList[0] = 0;
        asTokenList[0] = 0;
        return;
    }

    for(pCurrPos=sFileContent; 
        pStart=strstr(pCurrPos, TOKEN_START); 
        (*pStart)=(*pEnd)=0, pCurrPos=pEnd+strlen(TOKEN_END))
    {
        pEnd = strstr(pStart, TOKEN_END);
        asTextList[iTextIndex++] = pCurrPos;
        asTextList[iTextIndex] = 0;
        asTokenList[iTokenIndex++] = pStart+strlen(TOKEN_START);
        asTokenList[iTokenIndex] = 0;
    }
    iNumTokens = iTokenIndex;
    if(*pCurrPos)
    {
        asTextList[iTextIndex++] = pCurrPos;
        asTextList[iTextIndex] = 0;
    }
}

int CTokenReplacer::getNumTokens(void)
{
    return iNumTokens;
}

void CTokenReplacer::slurpFile(const char *sFileName, char **sBuffer)
{
    FILE *fp;

    if (fp = fopen(sFileName, "r"))
    {
        char sReadChunk[1024];
        int  iFileSize, iReadSize;

        *sBuffer = (char *) malloc(iFileSize=1024);
        **sBuffer = 0;
        while(0 < (iReadSize=fread(sReadChunk, 1, sizeof(sReadChunk)-1, fp)))
        {
            sReadChunk[iReadSize] = 0;
            appendAndExpand(sBuffer, sReadChunk, &iFileSize);
        }
    }
}


/**
 * test routine, uncomment to test
 */
/*
int main(int argc, char **argv)
{
    CTokenReplacer  *pTokenReplacer;
    char            sFileName[256];
    int             iOption, iIndex;
    char            **asTokenList;
    char            sInTokenVal[1024];
    char            *pTok;
    char            *asInTokVals[MAX_NUM_TOKENS];
    char            *pOutHTML;
    int             iOutHTMLLen;

    if (argc < 2)
    {
        printf("Enter file name: ");
        fflush(stdin);
        fflush(stdout);
        scanf("%s", sFileName);
    }
    else
    {
        strcpy(sFileName, argv[1]);
    }

    pTokenReplacer = new CTokenReplacer(sFileName);

    do
    {
        printf("1. Get token list\n");
        printf("2. Prepare HTML\n");
        printf("0. Quit\n");
        fflush(stdout);
        fflush(stdin);
        scanf("%d", &iOption);

        switch(iOption)
        {
        case 1:
            asTokenList = pTokenReplacer->getTokenList();
            for(iIndex=0; asTokenList[iIndex]; iIndex++)
            {
                printf("%s\n", asTokenList[iIndex]);
            }
            break;
        case 2:
            printf("Input tokens: ");
            fflush(stdout);
            fflush(stdin);
            gets(sInTokenVal);
            memset(asInTokVals, 0, sizeof(asInTokVals));
            for(iIndex=0, pTok=strtok(sInTokenVal, " "); pTok; pTok=strtok(0, " "), iIndex++)
            {
                asInTokVals[iIndex] = pTok;
            }
            pOutHTML = 0;
            iOutHTMLLen = 0;
            pTokenReplacer->prepareHTML((const char**)asInTokVals, &pOutHTML, &iOutHTMLLen);
            printf("\n[%s]\n", pOutHTML);
            break;
        }
    } while (iOption);

    return 0;
}
*/
